%----------------------------------
% Demo: Beamformer Design using Average and Worst-case Sidelobe
% Minimization
% by  Wing-Kin Ma and Jiaxian Pan and, March 2013
%
% Problem 1: average sidelobe minimization
%
% min w^H R w s.t. w^H a_des = 1
%
% where R = \int_{\theta \in \Omega} a(\theta) a^H(\theta) d\theta,
% \Omega is the stopband.
%
% Problem 2: worst-case sidelobe minimization
%
% min max_{\theta \in \Omega} |w^H a(\theta)|^2
% s.t. w^H a_des= 1
%

m= 40; % array size

angle_des= 10; % the desired angle, in degree
cut_off_angle= 5; % the cut-off angle, in degree
angle_res= 200; %no. of discretization points
a_des= steering_vec(angle_des*(pi/180),m);
theta_noi= [ (-pi/2):(pi/angle_res):(((angle_des-cut_off_angle)*(pi/180))) ];
theta_noi= [ theta_noi ((angle_des+cut_off_angle)*(pi/180)):(pi/angle_res):(pi/2) ];
L= length(theta_noi);

%----------------------------------
% average sidelobe minimization
R= zeros(m,m);
for i=1:length(theta_noi)
    a= steering_vec(theta_noi(i),m)/m;
    R= R+ a*(a');
end;
inv_R= inv(R);
w1= inv_R*a_des/real(a_des'*inv_R*a_des);

%----------------------------------
% worst-case sidelobe minimization
cvx_begin;
variable w2(m) complex;
variable t;

minimize t
subject to

w2'*a_des==1;

for i=1:L
    a= steering_vec(theta_noi(i),m);    
    square(w2'*a)<=t;
end;

cvx_end;


%----------------------------------
% legacy:
% I used to use sedumi to do worst-case sidelobe minimization
% You can see that we need more lines than CVX!
%
% %set up sedumi
% %1: desired angle constraint
% A_des= [ real(a_des)' imag(a_des)'; imag(a_des)' -real(a_des)'  ];
% At= [ A_des -A_des zeros(2,3*L) ]; bt= [1; 0 ];
% %2: sidelobe constraints
% for i=1:L
%     a= steering_vec(theta_noi(i),m);
%     A_noi= [ real(a)' imag(a)'; imag(a)' -real(a)'  ];
%     A_temp= [ A_noi -A_noi zeros(2,3*L) ];
%     A_temp(:,(4*m+(i-1)*3+(1:3)))= [ zeros(2,1) -eye(2) ];
%     At= [ At; A_temp ]; bt= [ bt; 0; 0 ];
% end;
% %3: equality with cone heights
% for i=2:L
%     A_temp= zeros(1,4*m+3*L);
%     A_temp(4*m+1)= 1; A_temp(4*m+(i-1)*3+1)= -1;
%     At= [ At; A_temp ]; bt= [ bt; 0 ];
% end;
% %4: set up the others
% clear Kt;
% Kt.l= [4*m]; Kt.q= [3];
% for i=2:L
%     Kt.q= [ Kt.q, 3 ];
% end;
% ct= zeros(4*m+3*L,1); ct(4*m+1)= 1;
% %run sedumi
% [xt,yt]= sedumi(sparse(At),sparse(bt),sparse(ct),Kt);
% wt= xt(1:2*m) - xt((2*m+1):(4*m));
% w2= wt(1:m)+ j*wt((m+1):(2*m));

%-----------------------
theta= -pi/2:(pi/1000):pi/2;
P1= zeros(length(theta),1); P2= zeros(length(theta),1);P3= zeros(length(theta),1);
% evaluate the beampatterns of the designed beamformers
for i=1:length(theta)
    P1(i)= abs(w1'*steering_vec(theta(i),m)).^2;
    P2(i)= abs(w2'*steering_vec(theta(i),m)).^2;
end;

figure(1); clf;
plot(theta*(2/pi*90),10*log10(P1),'linewidth',1); hold on;
plot(theta*(2/pi*90),10*log10(P2),'r','linewidth',1);
xlabel('Angle (Degree)'); ylabel('Magnitude (dB)');
set(gca,'XLim',[-90,90]);
legend('Average sidelobe min.','Worst-case sidelobe min.');